@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2018-2023 Hans Petter Jansson
+/* Copyright (C) 2018-2024 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -241,6 +241,11 @@ eval_symbol_error (const ChafaWorkCell *wcell,
         pair = eval->colors;
     }
 
+#ifdef HAVE_AVX2_INTRINSICS
+    if (chafa_have_avx2 ())
+        error = calc_error_avx2 (wcell->pixels, &pair, sym->mask_u32);
+    else
+#endif
 #ifdef HAVE_SSE41_INTRINSICS
     if (chafa_have_sse41 ())
         error = calc_error_sse41 (wcell->pixels, &pair, covp);
@@ -1920,6 +1925,7 @@ chafa_canvas_print_rows_strv (ChafaCanvas *canvas, ChafaTermInfo *term_info)
     }
 
     strv [gsa_len] = NULL;
+    g_free (gsa);
     return strv;
 }
 
