@@ -1,8 +1,8 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2018-2022 Hans Petter Jansson
+/* Copyright (C) 2018-2024 Hans Petter Jansson
  *
- * This file is part of Chafa, a program that turns images into character art.
+ * This file is part of Chafa, a program that shows pictures on text terminals.
  *
  * Chafa is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published
@@ -60,6 +60,7 @@ typedef enum
     CHAFA_SYMBOL_TAG_SEXTANT     = (1 << 22),
     CHAFA_SYMBOL_TAG_WEDGE       = (1 << 23),
     CHAFA_SYMBOL_TAG_LATIN       = (1 << 24),
+    CHAFA_SYMBOL_TAG_IMPORTED    = (1 << 25),
     CHAFA_SYMBOL_TAG_EXTRA       = (1 << 30),
     CHAFA_SYMBOL_TAG_BAD         = CHAFA_SYMBOL_TAG_AMBIGUOUS | CHAFA_SYMBOL_TAG_UGLY,
     CHAFA_SYMBOL_TAG_ALL         = ~(CHAFA_SYMBOL_TAG_EXTRA | CHAFA_SYMBOL_TAG_BAD)
