@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2020-2023 Hans Petter Jansson
+/* Copyright (C) 2020-2024 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -798,38 +798,21 @@ chafa_prepare_pixel_data_for_symbols (const ChafaPalette *palette,
 void
 chafa_sort_pixel_index_by_channel (guint8 *index, const ChafaPixel *pixels, gint n_pixels, gint ch)
 {
-    const gint gaps [] = { 57, 23, 10, 4, 1 };
-    gint g, i, j;
+    guint8 buckets [256] [64];
+    guint8 bucket_size [256] = { 0 };
+    gint i, j, k;
 
-    /* Since we don't care about stability and the number of elements
-     * is small and known in advance, use a simple in-place shellsort.
-     *
-     * Due to locality and callback overhead this is probably faster
-     * than qsort(), although admittedly I haven't benchmarked it.
-     *
-     * Another option is to use radix, but since we support multiple
-     * color spaces with fixed-point reals, we could get more buckets
-     * than is practical. */
+    g_assert (n_pixels <= 64);
 
-    for (g = 0; ; g++)
+    for (i = 0; i < n_pixels; i++)
     {
-        gint gap = gaps [g];
+        guint8 bucket = pixels [i].col.ch [ch];
+        buckets [bucket] [bucket_size [bucket]++] = i;
+    }
 
-        for (i = gap; i < n_pixels; i++)
-        {
-            guint8 ptemp = index [i];
-
-            for (j = i; j >= gap && pixels [index [j - gap]].col.ch [ch]
-                                  > pixels [ptemp].col.ch [ch]; j -= gap)
-            {
-                index [j] = index [j - gap];
-            }
-
-            index [j] = ptemp;
-        }
-
-        /* After gap == 1 the array is always left sorted */
-        if (gap == 1)
-            break;
+    for (i = 0, k = 0; i < 256; i++)
+    {
+        for (j = 0; j < bucket_size [i]; j++)
+            index [k++] = buckets [i] [j];
     }
 }
