@@ -1,8 +1,8 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2021-2022 Hans Petter Jansson
+/* Copyright (C) 2021-2024 Hans Petter Jansson
  *
- * This file is part of Chafa, a program that turns images into character art.
+ * This file is part of Chafa, a program that shows pictures on text terminals.
  *
  * Chafa is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published
@@ -85,6 +85,7 @@ const char chafa_ascii_dec_u8 [256] [4] =
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wunknown-warning-option"
 
+#pragma GCC diagnostic ignored "-Wuninitialized"
 #pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
 #pragma clang diagnostic ignored "-Wuninitialized"
 
@@ -126,3 +127,20 @@ chafa_format_dec_uint_0_to_9999 (char *dest, guint arg)
 
 #pragma clang diagnostic pop
 #pragma GCC diagnostic pop
+
+static gchar
+format_hex_digit (guchar n)
+{
+    g_assert (n < 16);
+    return (n < 10) ? '0' + n : 'a' - 10 + n;
+}
+
+gchar *
+chafa_format_dec_u16_hex (char *dest, guint16 arg)
+{
+    *(dest++) = format_hex_digit ((arg >> 12) & 0xf);
+    *(dest++) = format_hex_digit ((arg >> 8) & 0xf);
+    *(dest++) = format_hex_digit ((arg >> 4) & 0xf);
+    *(dest++) = format_hex_digit (arg & 0xf);
+    return dest;
+}
