@@ -1,8 +1,8 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2018-2022 Hans Petter Jansson
+/* Copyright (C) 2018-2024 Hans Petter Jansson
  *
- * This file is part of Chafa, a program that turns images into character art.
+ * This file is part of Chafa, a program that shows pictures on text terminals.
  *
  * Chafa is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published
@@ -1455,20 +1455,6 @@
             " X      "
             "X       ")
     },
-    {
-        /* Variant */
-        CHAFA_SYMBOL_TAG_BORDER | CHAFA_SYMBOL_TAG_DIAGONAL,
-        0x2571,
-        CHAFA_SYMBOL_OUTLINE_8X8 (
-            "      XX"
-            "     XXX"
-            "    XXX "
-            "   XXX  "
-            "  XXX   "
-            " XXX    "
-            "XXX     "
-            "XX      ")
-    },
     {
         CHAFA_SYMBOL_TAG_BORDER | CHAFA_SYMBOL_TAG_DIAGONAL,
         0x2572,
@@ -1482,19 +1468,6 @@
             "      X "
             "       X")
     },
-    {
-        CHAFA_SYMBOL_TAG_BORDER | CHAFA_SYMBOL_TAG_DIAGONAL,
-        0x2572,
-        CHAFA_SYMBOL_OUTLINE_8X8 (
-            "XX      "
-            "XXX     "
-            " XXX    "
-            "  XXX   "
-            "   XXX  "
-            "    XXX "
-            "     XXX"
-            "      XX")
-    },
     {
         CHAFA_SYMBOL_TAG_BORDER | CHAFA_SYMBOL_TAG_DIAGONAL,
         0x2573,
