@@ -7,5 +7,5 @@ extensions="$(get_supported_loaders)"
 
 for ext in $extensions; do
     [ "x${ext}" = "ximagemagick" ] && continue
-    run_cmd_single_file "$tool -f sixel --threads 12 --animate no" "good/pixel.$ext"
+    run_cmd_single_file "$tool -f sixel --threads 12 --animate no" "good/pixel.$ext" || exit $?
 done
