@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2023 Hans Petter Jansson
+/* Copyright (C) 2024 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -40,6 +40,7 @@
 #include "tiff-loader.h"
 #include "webp-loader.h"
 #include "avif-loader.h"
+#include "jxl-loader.h"
 
 typedef enum
 {
@@ -52,6 +53,7 @@ typedef enum
     LOADER_TYPE_WEBP,
     LOADER_TYPE_AVIF,
     LOADER_TYPE_SVG,
+    LOADER_TYPE_JXL,
 
     LOADER_TYPE_LAST
 }
@@ -189,6 +191,20 @@ loader_vtable [LOADER_TYPE_LAST] =
         (gint (*) (gpointer)) avif_loader_get_frame_delay
     },
 #endif
+#ifdef HAVE_JXL
+    [LOADER_TYPE_JXL] =
+    {
+    "JXL",
+    (gpointer (*)(gpointer)) jxl_loader_new_from_mapping,
+        (gpointer (*)(gconstpointer)) NULL,
+    (void (*)(gpointer)) jxl_loader_destroy,
+    (gboolean (*)(gpointer)) jxl_loader_get_is_animation,
+    (void (*)(gpointer)) jxl_loader_goto_first_frame,
+    (gboolean (*)(gpointer)) jxl_loader_goto_next_frame,
+    (gconstpointer (*) (gpointer, gpointer, gpointer, gpointer, gpointer)) jxl_loader_get_frame_data,
+    (gint (*) (gpointer)) jxl_loader_get_frame_delay
+    },
+#endif
 };
 
 struct MediaLoader
