@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2018-2023 Hans Petter Jansson
+/* Copyright (C) 2018-2024 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -42,7 +42,7 @@ typedef enum
 RotationType;
 
 RotationType invert_rotation (RotationType rot);
-void rotate_image (guchar **src, guint *width, guint *height, guint *rowstride,
+void rotate_image (gpointer *src, guint *width, guint *height, guint *rowstride,
                    guint n_channels, RotationType rot);
 
 G_END_DECLS
